import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { MarcaService } from '../marca/marca.service';
import { ModeloService } from '../modelo/modelo.service';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private institucionService: InstitucionService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private marcaService: MarcaService,
    private modeloService: ModeloService,
    private moduloService: ModuloService,
  ) {}

  async create(
    id_modulo: number,
    id_tipo_carrito: number,
    marca: string,
    modelo: string,
    // id_marca: number,
    // id_modelo: number,
  ) {
    // const marca = await this.marcaService.findById(id_marca);
    // const modelo = await this.modeloService.findById(id_modelo);
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findTipoCarritoById(
        id_tipo_carrito,
      );

    return this.repository
      .count({ modulo, tipoCarrito })
      .then((carritos) =>
        this.repository.save(
          this.repository.create({
            carrito: `C${carritos >= 9 ? '' : '0'}${carritos + 1}`,
            marca,
            modelo,
            modulo,
            tipoCarrito,
          }),
        ),
      )
      .then((_) => ({
        message: 'Se creó correctamente un nuevo carrito para este módulo.',
      }));
  }

  async findAll(filtros: {
    pagina: string;
    activo?: string;
    carrito?: string;
    id_institucion?: string;
    id_marca?: string;
    id_modelo?: string;
    id_modulo?: string;
    id_tipo_carrito?: string;
  }) {
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    // const marca = filtros.id_marca
    //   ? await this.marcaService.findById(parseInt(filtros.id_marca))
    //   : null;
    // const modelo = filtros.id_modelo
    //   ? await this.modeloService.findById(parseInt(filtros.id_modelo))
    //   : null;
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const query = this.repository
      .createQueryBuilder('c')
      .innerJoinAndSelect('c.modulo', 'm')
      .innerJoinAndSelect('c.tipoCarrito', 'tc')
      .innerJoinAndSelect('m.institucion', 'i')
      .orderBy('i.institucion')
      .addOrderBy('m.modulo')
      .addOrderBy('tc.tipo_carrito')
      .addOrderBy('c.carrito')
      .skip((parseInt(filtros.pagina) - 1) * 25)
      .take(25);

    if (filtros.activo === 'true') query.andWhere('c.activo = 1');
    if (filtros.activo === 'false') query.andWhere('c.activo = 0');
    if (filtros.carrito)
      query.andWhere('c.carrito LIKE :carrito', {
        carrito: `%${filtros.carrito}%`,
      });
    if (institucion)
      query.andWhere('i.id_institucion = :id_institucion', {
        id_institucion: institucion.id_institucion,
      });
    if (modulo)
      query.andWhere('m.id_modulo = :id_modulo', {
        id_modulo: modulo.id_modulo,
      });
    if (tipoCarrito)
      query.andWhere('tc.id_tipo_carrito = :id_tipo_carrito', {
        id_tipo_carrito: tipoCarrito.id_tipo_carrito,
      });
    return query.getManyAndCount();
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este id carrito.');
      return carrito;
    });
  }

  async findCarrito(
    id_modulo: number | Modulo,
    id_tipo_carrito: number | TipoCarrito,
    carrito: string,
    validarNoExiste = true,
  ) {
    const modulo =
      typeof id_modulo === 'number'
        ? await this.moduloService.findById(id_modulo)
        : id_modulo;
    const tipoCarrito =
      typeof id_tipo_carrito === 'number'
        ? await this.institucionTipoCarritoService.findTipoCarritoById(
            id_tipo_carrito,
          )
        : id_tipo_carrito;

    return this.repository
      .findOne({ carrito, modulo, tipoCarrito })
      .then((carrito) => {
        if (validarNoExiste && !carrito)
          throw new ConflictException('No existe este carrito.');
        return carrito;
      });
  }

  async update(
    attrs: Partial<Carrito>,
    // id_marca?: number,
    // id_modelo?: number,
  ) {
    // const marca = id_marca
    //   ? typeof id_marca === 'number'
    //     ? await this.marcaService.findById(id_marca)
    //     : id_marca
    //   : null;
    // const operador = id_operador
    //   ? typeof id_operador === 'number'
    //     ? await this.operadorService.findById(id_operador)
    //     : id_operador
    //   : null;

    return this.findById(attrs.id_carrito)
      .then(async (carrito) => {
        Object.assign(carrito, attrs);
        // if (marca) carrito.marca = marca;
        // if (modelo) carrito.modelo = modelo;
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
